﻿------------------------------------------------------------------------------------------------------
-- Initialise the variable we will be working with
------------------------------------------------------------------------------------------------------
InstanceInfo = LibStub("AceAddon-3.0"):NewAddon("Broker_InstanceInfo", "AceEvent-3.0")

-- To allow this to quickly be used for multiple addons
local addon = InstanceInfo;

-- Various libs we'll be working with
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local QTC = LibStub("LibQTip-1.0")
local L = LibStub("AceLocale-3.0"):GetLocale("InstanceInfo");
local AceConfig = LibStub("AceConfigDialog-3.0");

-- Locals we'll need
local tooltip
local DO



------------------------------------------------------------------------------------------------------
-- Find the hex colour for each class
------------------------------------------------------------------------------------------------------
local function GetHexColor(class)
	return string.format("%02x%02x%02x", 
		RAID_CLASS_COLORS[class].r * 255,
		RAID_CLASS_COLORS[class].g * 255,
		RAID_CLASS_COLORS[class].b * 255
	);
end

------------------------------------------------------------------------------------------------------
-- Colouring functions
------------------------------------------------------------------------------------------------------
local function ColourText(colour, text)
	local colours = {
		red	= "ffff0000",
		yellow	= "ffffff00",
		green	= "ff00ff00",
		white	= "ffffffff"
	};
	return "|c" .. colours[colour] .. text .. "|r"
end

------------------------------------------------------------------------------------------------------
-- Function to get and display saved instances for the specified player
------------------------------------------------------------------------------------------------------
local function ShowSavedInstanceInfo(player)
	local info = addon.db.realm.info[player];

	if #(info.instances) == 0 then
		-- We have no info for this character
		return;
	end
	
	-- Saved Instances header
	local y = tooltip:AddLine();
	tooltip:SetCell(y, 1, "[|cff"..GetHexColor(info.engClass)..info.player.."|r]");
	tooltip:SetCell(y, 2, L["Zone"], "LEFT");
	tooltip:SetCell(y, 3, L["Raid ID"], "LEFT");
	tooltip:SetCell(y, 4, L["Reset Time"], "LEFT");

	local nMax = #(info.instances);
	for i = 1, nMax do
		local instance = info.instances[i];
		if (instance ~= nil) then
			if (time() <= (info.curtime + instance.remaining)) then
				
				local name = instance.name
				if instance.difficulty > 1 then
					name = instance.name.." ("..DUNGEON_DIFFICULTY2..")"
				end
				
				-- The actual display
				local y = tooltip:AddLine()
				tooltip:SetCell(y, 2, name, "LEFT")
				tooltip:SetCell(y, 3, instance.ID, "LEFT")
				tooltip:SetCell(y, 4, SecondsToTime((info.curtime + instance.remaining) - time(), true, false, 4), "LEFT")
			else
				-- This timer expired
				addon.db.realm.info[player].instances[i] = nil;
				addon:UpdateBrokerText();
			end
		end
	end
end

------------------------------------------------------------------------------------------------------
-- Event that fires:
--	when we leave the world
--	when the instance we are in gets updated
--	when the instance we are in starts up
--	when the instance we are in is done starting
------------------------------------------------------------------------------------------------------
local function OnInstanceEvent()
	addon:SaveInstanceInfo();
end

------------------------------------------------------------------------------------------------------
-- Fetches the number of saved instances
------------------------------------------------------------------------------------------------------
local function GetSavedInstances()
	local numInstances = 0;

	if (addon.db.profile.showSelf) then
		local nMax = GetNumSavedInstances();
		for i = 1, nMax do
			local instance = addon.db.realm.info[UnitName("player")].instances[i];
			if (instance ~= nil) then
				numInstances = numInstances + 1;
			end
		end
	else
		for player, playerinfo in pairs(addon.db.realm.info) do
			if (playerinfo ~= nil) then
				local nMax = #(playerinfo.instances);
				for i = 1, nMax do
					local instance = playerinfo.instances[i];
					if (instance ~= nil) then
						numInstances = numInstances + 1;
					end
				end
			end
		end
	end
	
	return numInstances
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is first loaded
------------------------------------------------------------------------------------------------------
function addon:OnInitialize()
	-- Register the database
	self.db = LibStub("AceDB-3.0"):New("InstanceInfoFuDB", self.defaults);
	self.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	-- Create options table and stuff
	LibStub("AceConfig-3.0"):RegisterOptionsTable("InstanceInfo", self.options, "instanceinfo")
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("InstanceInfo", "InstanceInfo")

	DO = LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("InstanceInfo", {
		type = "data source",
		icon = "Interface\\Icons\\INV_Misc_PocketWatch_01",
		label = "InstanceInfo",
		text = "InstanceInfo",
		OnClick = function(frame, button)
			if button == "LeftButton" then
				-- Open the Raid frame
				ToggleFriendsFrame(5);
			elseif button == "RightButton" then
				InterfaceOptionsFrame_OpenToCategory("InstanceInfo")
			end
		end,
		OnEnter = function(frame)
			tooltip = QTC:Acquire("Broker_InstanceInfoTooltip", 4, "LEFT", "LEFT", "CENTER","RIGHT")
			tooltip:SmartAnchorTo(frame)
			tooltip:SetAutoHideDelay(0.25, frame)
			addon:ShowTooltip()
		end,
		OnLeave = function()
			return
		end,
	})
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is enabled
------------------------------------------------------------------------------------------------------
function addon:OnEnable()
	-- Register for important events
	addon:RegisterEvent("PLAYER_ENTERING_WORLD",	OnInstanceEvent);
	addon:RegisterEvent("PLAYER_LEAVING_WORLD",	OnInstanceEvent);
	addon:RegisterEvent("UPDATE_INSTANCE_INFO",	OnInstanceEvent);
	addon:RegisterEvent("INSTANCE_BOOT_START",	OnInstanceEvent);
	addon:RegisterEvent("INSTANCE_BOOT_STOP",	OnInstanceEvent);

	-- Ask for instance info
	RequestRaidInfo();
	
	-- Update text
	self:UpdateBrokerText();
end

------------------------------------------------------------------------------------------------------
-- What happens when the addon is disabled
------------------------------------------------------------------------------------------------------
function addon:OnDisable()
	-- Remove registration for our events
	addon:UnregisterEvent("PLAYER_LEAVING_WORLD")
	addon:UnregisterEvent("UPDATE_INSTANCE_INFO")
	addon:UnregisterEvent("INSTANCE_BOOT_START")
	addon:UnregisterEvent("INSTANCE_BOOT_STOP")
end

------------------------------------------------------------------------------------------------------
-- Update saved text
------------------------------------------------------------------------------------------------------
function addon:OnUpdate()	
	-- Update Broker text
	self:UpdateBrokerText();
end

------------------------------------------------------------------------------------------------------
-- Broker text update code
------------------------------------------------------------------------------------------------------
function addon:UpdateBrokerText()
	local numInstances = GetSavedInstances();
	
	if (numInstances == 0) then
		-- We're not saved to anything
		DO.text = ColourText("green", L["No Saved Instances"]);
	else
		DO.text = ColourText("green", L["Saved Instances"]..":").." "..numInstances;
	end
end

------------------------------------------------------------------------------------------------------
-- Registers the tooltip (thanks to Zhinjio from Broker_HitCrit whose code taught me how to do this)
------------------------------------------------------------------------------------------------------
function addon:ShowTooltip()
	-- Begin defining some local variables
	local numInstances	= GetSavedInstances();
	local headerFont	= CreateFont("instanceinfoHeaderFont");

	-- Sort font objects
	instanceinfoHeaderFont:CopyFontObject(GameTooltipHeaderText)
	instanceinfoHeaderFont:SetFont(instanceinfoHeaderFont:GetFont(), 15)

	-- Make sure we don't have any other tooltips
	tooltip:Clear()

	-- Set the scale of the tooltip
	tooltip:SetScale(1)

	-- Add header
	tooltip:SetCell(tooltip:AddLine(), 1, "InstanceInfo", instanceinfoHeaderFont, "CENTER", 4)
	tooltip:AddLine(" "); -- Line separator

	if (numInstances == 0) then
		-- We don't have any saved instances
		tooltip:SetCell(tooltip:AddLine(), 1, ColourText("green", L["No Saved Instances"]), "CENTER", 4)
	else
		-- Display all our saved instances
		if (not self.db.profile.showSelf) then
			for player, playerinfo in pairs(self.db.realm.info) do
				if (playerinfo ~= nil and playerinfo ~= {}) then
					ShowSavedInstanceInfo(player);
				end
			end
		else
			-- We're only showing ourselves
			ShowSavedInstanceInfo(UnitName("player"));
		end
	end
	
	-- Add menu items
	tooltip:AddLine(" ") -- Line separator
	tooltip:SetCell(tooltip:AddLine(), 1, ColourText("yellow", L["Left-click|r to open the Raid panel"]),	"LEFT", 4)
	tooltip:SetCell(tooltip:AddLine(), 1, ColourText("yellow", L["Right-click|r to open the options menu"]),	"LEFT", 4)

	tooltip:Show();
end

------------------------------------------------------------------------------------------------------
-- Instance information saving
------------------------------------------------------------------------------------------------------
function addon:SaveInstanceInfo()
	if (UnitLevel("player") < tonumber(self.db.profile.minLevel)) then
		-- We don't want to store instance info for low-level players
		return;
	end

	local player = UnitName("player");
	if (self.db.realm.info[player] == nil) then
		self.db.realm.info[player] = {};
	end
	
	-- Store a bunch of fun stuff
	self.db.realm.info[player].player	= player;
	self.db.realm.info[player].engClass	= select(2, UnitClass("player"));
	self.db.realm.info[player].curtime	= time();
	
	-- Store name of instances we are saved to
	self.db.realm.info[player].instances	= {};

	local numInstances = GetNumSavedInstances();
	for i = 1, numInstances do
		local instance = {};
		instance.name, instance.ID, instance.remaining, instance.difficulty = GetSavedInstanceInfo(i);

		if (instance.remaining > 0) then
			-- Only save it for instances that aren't expired
			self.db.realm.info[player].instances[i] = instance;
		end
	end

	table.sort(self.db.realm.info[player].instances, function(a, b) return a.name < b.name end);
	table.sort(self.db.realm.info, function(a,b) return a.player < b.player; end);
	
	-- Update our text
	self:UpdateBrokerText();

	return true;
end